#!/usr/bin/env python

"""
(c) ORIGINAL FROM: 2013 Marius Retegan
License: BSD-2-Clause
Description: Create a .cube file of the electrostatic potential using ORCA.
Run: python mep.py basename npoints (e.g. python mep.py water 40)
Arguments: basename - file name without the extension;
                      this should be the same for the .gbw and .scfp.
           npoints  - number of grid points per side
                      (80 should be fine)
Dependencies: numpy


Addapted for GFN-xTB by Fabian Bohle
"""
def coord2xyz(inputfile):
    """convert TURBOMOLE coordfile to xyz"""
    bohr2ang = 0.52917721067
    with open(inputfile, 'r') as f:
        coord = f.readlines()
        x = []
        y = []
        z = []
        atom = []
        for line in coord[1:-1]:
            x.append(float(line.split()[0])*bohr2ang)
            y.append(float(line.split()[1])*bohr2ang)
            z.append(float(line.split()[2])*bohr2ang)
            atom.append(str(line.split()[3]))
        for item in atom:
            if len(item) == 1:
                atom[atom.index(item)] = item.replace(item[0], item[0].upper())
            if len(item) >= 2:
                atom[atom.index(item)] = item.replace(item, item[0].upper()+item[1:].lower())
        #natoms = int(len(coord[1:-1]))
        return atom, np.array(x), np.array(y), np.array(z)

def read_xyz(xyz):
    x = []
    y = []
    z = []
    atom = []
    f = open(xyz, "r")
    f.next()
    f.next()
    for line in f:
        data = line.split()
        atom.append(data[0])
        x.append(float(data[1]))
        y.append(float(data[2]))
        z.append(float(data[3]))
    f.close()
    for item in atom:
        if len(item) == 1:
            atom[atom.index(item)] = item.replace(item[0], item[0].upper())
        if len(item) >= 2:
            atom[atom.index(item)] = item.replace(item, item[0].upper()+item[1:].lower())
    return atom, np.array(x), np.array(y), np.array(z)


def read_vpot(vpot):
    v = []
    f = open(vpot, "r")
    for line in f:
        data = line.split()
        v.append(float(data[3]))
    f.close()
    return np.array(v)

def cml():
    """ Get args object from commandline interface.
        Needs argparse module."""
    descr='''
     _____________________________________________________
    |Creating cubefile with ESP data from GFN-xTB         |
    |                                                     |
    |usage: cube.py -i file.xyz -n #points                |
    |_____________________________________________________|'''

    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter, usage=argparse.SUPPRESS)
    # Add more options
    parser.add_argument("-i", "--input",
                        dest="file",
                        action="store",
                        required=True,
                        help="inputfile coord")
    parser.add_argument("-n", "--number",
                        dest="points",
                        action="store",
                        required=True,
                        help="number of points along axis")
    parser.add_argument("--xtbpath",
                        dest="xtbpath",
                        action="store",
                        required=False,
                        default="xtb",
                        help="optional path to special user-xtb")
    args = parser.parse_args()
    return args;




if __name__ == "__main__":
    import argparse
    import sys
    from os.path import splitext
    import subprocess
    import numpy as np
    
    
    args = cml()
    basename, ext  = splitext(args.file)
    npoints = int(args.points)

    ang_to_au = 1.0 / 0.5291772083

    elements = [None,
         "H", "He",
         "Li", "Be",
         "B", "C", "N", "O", "F", "Ne",
         "Na", "Mg",
         "Al", "Si", "P", "S", "Cl", "Ar",
         "K", "Ca",
         "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn",
         "Ga", "Ge", "As", "Se", "Br", "Kr",
         "Rb", "Sr",
         "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd",
         "In", "Sn", "Sb", "Te", "I", "Xe",
         "Cs", "Ba",
         "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb",
         "Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg",
         "Tl", "Pb", "Bi", "Po", "At", "Rn",
         "Fr", "Ra",
         "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No",
         "Lr", "Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg", "Uub"]
    

    

    ### test whether coord or xyz:
    with open(args.file, 'r') as test:
        check = test.readline()
    if check.split()[0] == '$coord':
        atoms, x, y, z = coord2xyz(args.file)
        print(atoms)
    else:
        atoms, x, y, z = read_xyz(args.file)
    natoms = len(atoms)

    extent = 7.0
    xmin = x.min() * ang_to_au - extent
    xmax = x.max() * ang_to_au + extent
    ymin = y.min() * ang_to_au - extent
    ymax = y.max() * ang_to_au + extent
    zmin = z.min() * ang_to_au - extent
    zmax = z.max() * ang_to_au + extent

    mep_inp = open("esp_coord", "w")
    for ix in np.linspace(xmin, xmax, npoints, True):
        for iy in np.linspace(ymin, ymax, npoints, True):
            for iz in np.linspace(zmin, zmax, npoints, True):
                mep_inp.write("{0:12.6f} {1:12.6f} {2:12.6f}  1 0.3\n".format(ix, iy, iz))
    mep_inp.close()
    
    print('Executing GFN2-xTB ESP calculation!')
    subprocess.check_call([args.xtbpath, args.file ,"-esp"])

    vpot = read_vpot("xtb_esp.dat")
    cube = open("esp.cube", "w")
    cube.write("Generated with GFN2-xTB\n")
    cube.write("Electrostatic potential for " + basename + "\n")
    cube.write("{0:5d}{1:12.6f}{2:12.6f}{3:12.6f}\n".format(
        len(atoms), xmin, ymin, zmin))
    cube.write("{0:5d}{1:12.6f}{2:12.6f}{3:12.6f}\n".format(
        npoints, (xmax - xmin) / float(npoints - 1), 0.0, 0.0))
    cube.write("{0:5d}{1:12.6f}{2:12.6f}{3:12.6f}\n".format(
        npoints, 0.0, (ymax - ymin) / float(npoints - 1), 0.0))
    cube.write("{0:5d}{1:12.6f}{2:12.6f}{3:12.6f}\n".format(
        npoints, 0.0, 0.0, (zmax - zmin) / float(npoints - 1)))
    for i, atom in enumerate(atoms):
        index = elements.index(atom)
        cube.write("{0:5d}{1:12.6f}{2:12.6f}{3:12.6f}{4:12.6f}\n".format(
            index, 0.0, x[i] * ang_to_au, y[i] * ang_to_au, z[i] * ang_to_au))

    #m = 0
    n = 0
    vpot = np.reshape(vpot, (npoints, npoints, npoints))
    for ix in range(npoints):
        for iy in range(npoints):
            for iz in range(npoints):
                cube.write("{:22.14f}".format(vpot[ix][iy][iz]))
               # m += 1
                n += 1
                if (n > 5):
                    cube.write("\n")
                    n = 0
            #if n != 0:
            #    cube.write("\n")
            #    n = 0
    cube.close()
